@@ -37,7 +37,7 @@ NULL
 #' @template param-origin
 #' @template param-dest
 #'
-#' @return A numeric vector or a single numeric value
+#' @return See Details
 #'
 #' @example inst/examples/example.R
 #'
@@ -55,6 +55,9 @@ setMethod(
   "cond_passage",
   signature(samc = "samc", origin = "missing", dest = "location"),
   function(samc, dest) {
+    if (samc@clumps == -1)
+      warning("Unknown number of clumps in data. If the function crashes, it may be due to the transition matrix being discontinuous.", call. = FALSE)
+
     if (samc@clumps > 1)
       stop("This function cannot be used with discontinuous data", call. = FALSE)
 
@@ -63,7 +66,7 @@ setMethod(
 
     dest <- .process_locations(samc, dest)
 
-    Q <- samc@p[-nrow(samc@p), -nrow(samc@p)]
+    Q <- samc$q_matrix
     qj <- Q[-dest, dest]
     Qj <- Q[-dest, -dest]
 
